<?php namespace EscapeWork\Manager\Providers;

use EscapeWork\Manager\Facades\ManagerAuth;
use EscapeWork\Manager\Permission;
use EscapeWork\Manager\Validators\PasswordMatchValidator;
use Illuminate\Support\ServiceProvider;
use Illuminate\Foundation\AliasLoader;

class ManagerServiceProvider extends ServiceProvider
{

    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        /**
         * Indicates package root folder
         */
        $root = __DIR__ . '/../..';

        # publiishing files
        $this->publishes([
            $root . '/config/manager.php'  => config_path('manager.php'),
            $root . '/resources/assets'    => public_path('packages/manager'),
            $root . '/database/migrations' => 'database/migrations',
        ]);

        # loading views and translations from resources
        $this->loadViewsFrom($root . '/resources/views', 'manager');
        $this->loadTranslationsFrom($root . '/resources/translations', 'manager');
        $this->loadValidators();

        $this->createConfigurations();
    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->registerMiddlewares();

        $this->registerPermission();
        $this->createAliases();

        $this->mergeConfigFrom(
            __DIR__ . '/../../config/manager.php', 'manager'
        );

        $this->callProviders();
        $this->registerCommands();
    }

    protected function registerMiddlewares()
    {
        $kernel = new \EscapeWork\Manager\Http\Kernel;

        foreach ($kernel->getMiddlewares() as $key => $middleware) {
            $this->app->router->middleware($key, $middleware);
        }
    }

    /**
     * Registering the permission check class
     */
    protected function registerPermission()
    {
        $this->app['escapework.manager.permission'] = $this->app->share(function($app)
        {
            $user = ManagerAuth::user();

            $permission = $app->make('EscapeWork\Manager\Permission', [$user]);
            $permission->fetchAllowedActions();

            return $permission;
        });
    }

    /**
     * Calling the providers needed
     */
    protected function callProviders()
    {
        $this->app->register('EscapeWork\Manager\Providers\RouteServiceProvider');
        $this->app->register('EscapeWork\Manager\Providers\UserServiceProvider');
        $this->app->register('EscapeWork\Manager\Providers\PasswordServiceProvider');
        $this->app->register('EscapeWork\Manager\Providers\ResponseServiceProvider');
        $this->app->register('Illuminate\Html\HtmlServiceProvider');

        if ($this->app['request']->is(config('manager.url') . '/*')) {
            # handling manager errors
            $this->app->singleton(
                'Illuminate\Contracts\Debug\ExceptionHandler',
                'EscapeWork\Manager\Exceptions\Handler'
            );

            # off por enquanto
            # $this->app->register('EscapeWork\Manager\Providers\EventServiceProvider');
        }
    }

    protected function createAliases()
    {
        $this->app->booting(function()
        {
            $loader = AliasLoader::getInstance();

            $loader->alias('Check', 'EscapeWork\Manager\Facades\Check');
            $loader->alias('Form', 'Illuminate\Html\FormFacade');
            $loader->alias('Html', 'Illuminate\Html\HtmlFacade');
        });
    }

    /**
     * Registrando os comandos
     */
    public function registerCommands()
    {
        $this->commands(
            'EscapeWork\Manager\Console\ConfigureCommand',
            'EscapeWork\Manager\Console\CreateUserCommand',
            'EscapeWork\Manager\Console\UpdatePasswordCommand',

            'EscapeWork\Manager\Console\CrudGeneratorCommand',
            'EscapeWork\Manager\Console\ControllerMakeCommand',
            'EscapeWork\Manager\Console\RequestMakeCommand',
            'EscapeWork\Manager\Console\ModelMakeCommand',
            'EscapeWork\Manager\Console\RepositoryMakeCommand',
            'EscapeWork\Manager\Console\CommandMakeCommand',
            'EscapeWork\Manager\Console\ViewsMakeCommand'
        );
    }

    /**
     * Criando as configurações em tempo de execução
     */
    protected function createConfigurations()
    {
        $this->app['config']->set('manager.packages', []);

        $this->app['config']['manager.permissions.pages'] = array_merge(
            $this->app['config']['manager.permissions.pages'],
            [
                'users' => [
                    'title'   => trans('manager::users.title'),
                    'actions' => [], // ex: create, read, update, delete, etc
                ],

                'roles' => [
                    'title'   => trans('manager::roles.title'),
                    'actions' => [], // ex: create, read, update, delete, etc
                ],
            ]
        );
    }

        /**
     * Resolving validators
     * @return void
     */
    protected function loadValidators()
    {
        $this->app->validator->extend('password_match', 'EscapeWork\Manager\Validators\PasswordMatchValidator@validate');
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return [
            'escapework.manager.permission',
        ];
    }
}
